@@ -1,7 +1,4 @@
-__all__ = [
-    'KilledError', 'UnsuccessfulResult', 'InvalidStateError',
-    'PersistenceError'
-]
+__all__ = ['KilledError', 'UnsuccessfulResult', 'InvalidStateError', 'PersistenceError']
 
 
 class KilledError(Exception):
@@ -24,10 +21,6 @@ class ProcessExcepted(Exception):
     pass
 
 
-class Interrupted(Exception):
-    pass
-
-
 class InvalidStateError(Exception):
     pass
 
