@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 
+from __future__ import absolute_import
 import abc
 import contextlib
 import functools
@@ -18,6 +19,7 @@ import yaml
 from .process_listener import ProcessListener
 from .process_spec import ProcessSpec
 from .utils import protected
+from . import communications
 from . import exceptions
 from . import futures
 from . import base
@@ -68,14 +70,11 @@ class ProcessStateMachineMeta(abc.ABCMeta, state_machine.StateMachineMeta):
 
 
 # Make ProcessStateMachineMeta instances (classes) YAML - able
-yaml.representer.Representer.add_representer(
-    ProcessStateMachineMeta, yaml.representer.Representer.represent_name)
+yaml.representer.Representer.add_representer(ProcessStateMachineMeta, yaml.representer.Representer.represent_name)
 
 
 @persistence.auto_persist('_pid', '_CREATION_TIME', '_future', '_paused', '_status', '_pre_paused_status')
-class Process(
-    with_metaclass(ProcessStateMachineMeta, StateMachine,
-                   persistence.Savable)):
+class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.Savable)):
     """
     The Process class is the base for any unit of work in plumpy.
 
@@ -208,12 +207,7 @@ class Process(
         base.call_with_super_check(process.init)
         return process
 
-    def __init__(self,
-                 inputs=None,
-                 pid=None,
-                 logger=None,
-                 loop=None,
-                 communicator=None):
+    def __init__(self, inputs=None, pid=None, logger=None, loop=None, communicator=None):
         """
         The signature of the constructor should not be changed by subclassing
         processes.
@@ -242,8 +236,7 @@ class Process(
         self._paused = None
 
         # Input/output
-        self._raw_inputs = None if inputs is None else utils.AttributesFrozendict(
-            inputs)
+        self._raw_inputs = None if inputs is None else utils.AttributesFrozendict(inputs)
         self._pid = pid
         self._parsed_inputs = None
         self._outputs = {}
@@ -254,7 +247,10 @@ class Process(
         self._future = persistence.SavableFuture()
         self.__event_helper = utils.EventHelper(ProcessListener)
         self._logger = logger
-        self._communicator = communicator
+        if communicator is None:
+            self._communicator = None
+        else:
+            self._communicator = communications.CommunicatorWrapper(communicator, self._loop)
 
     @base.super_check
     def init(self):
@@ -263,28 +259,24 @@ class Process(
         self._terminated_callbacks = []
 
         if self._communicator is not None:
-            self._communicator.add_rpc_subscriber(
-                self.message_receive, identifier=str(self.pid))
+            self._communicator.add_rpc_subscriber(self.message_receive, identifier=str(self.pid))
+
         if not self._future.done():
 
             def try_killing(future):
                 if future.cancelled():
                     if not self.kill('Killed by future being cancelled'):
-                        self.logger.warning(
-                            "Failed to kill process on future cancel")
+                        self.logger.warning("Failed to kill process on future cancel")
 
             self._future.add_done_callback(try_killing)
 
     def _setup_event_hooks(self):
-        self.add_state_event_callback(
-            state_machine.StateEventHook.ENTERING_STATE,
-            lambda _s, _h, state: self.on_entering(state))
-        self.add_state_event_callback(
-            state_machine.StateEventHook.ENTERED_STATE,
-            lambda _s, _h, from_state: self.on_entered(from_state))
-        self.add_state_event_callback(
-            state_machine.StateEventHook.EXITING_STATE,
-            lambda _s, _h, _state: self.on_exiting())
+        self.add_state_event_callback(state_machine.StateEventHook.ENTERING_STATE,
+                                      lambda _s, _h, state: self.on_entering(state))
+        self.add_state_event_callback(state_machine.StateEventHook.ENTERED_STATE,
+                                      lambda _s, _h, from_state: self.on_entered(from_state))
+        self.add_state_event_callback(state_machine.StateEventHook.EXITING_STATE,
+                                      lambda _s, _h, _state: self.on_exiting())
 
     @property
     def creation_time(self):
@@ -351,11 +343,7 @@ class Process(
 
     def launch(self, process_class, inputs=None, pid=None, logger=None):
         process = process_class(
-            inputs=inputs,
-            pid=pid,
-            logger=logger,
-            loop=self.loop(),
-            communicator=self._communicator)
+            inputs=inputs, pid=pid, logger=logger, loop=self.loop(), communicator=self._communicator)
         self.call_soon_external(process.step_until_terminated)
         return process
 
@@ -472,8 +460,7 @@ class Process(
         # Make sure execute is a coroutine
         coro = utils.ensure_coroutine(fn)
         result = yield tornado.stack_context.run_with_stack_context(
-            tornado.stack_context.StackContext(self._process_scope),
-            functools.partial(coro, *args, **kwargs))
+            tornado.stack_context.StackContext(self._process_scope), functools.partial(coro, *args, **kwargs))
         raise gen.Return(result)
 
     # endregion
@@ -494,16 +481,13 @@ class Process(
 
         # Inputs/outputs
         if self.raw_inputs is not None:
-            out_state[BundleKeys.INPUTS_RAW] = self.encode_input_args(
-                self.raw_inputs)
+            out_state[BundleKeys.INPUTS_RAW] = self.encode_input_args(self.raw_inputs)
 
         if self.inputs is not None:
-            out_state[BundleKeys.INPUTS_PARSED] = self.encode_input_args(
-                self.inputs)
+            out_state[BundleKeys.INPUTS_PARSED] = self.encode_input_args(self.inputs)
 
         if self.outputs:
-            out_state[BundleKeys.OUTPUTS] = self.encode_input_args(
-                self.outputs)
+            out_state[BundleKeys.OUTPUTS] = self.encode_input_args(self.outputs)
 
     @protected
     def load_instance_state(self, saved_state, load_context):
@@ -525,8 +509,8 @@ class Process(
 
         self._state = self.recreate_state(saved_state['_state'])
 
-        if 'communicator' in load_context:
-            self._communicator = load_context.communicator
+        if 'communicator' in load_context and load_context.communicator is not None:
+            self._communicator = communications.CommunicatorWrapper(load_context.communicator, self._loop)
 
         if 'logger' in load_context:
             self._logger = load_context.logger
@@ -537,15 +521,13 @@ class Process(
 
         # Inputs/outputs
         try:
-            decoded = self.decode_input_args(
-                saved_state[BundleKeys.INPUTS_RAW])
+            decoded = self.decode_input_args(saved_state[BundleKeys.INPUTS_RAW])
             self._raw_inputs = utils.AttributesFrozendict(decoded)
         except KeyError:
             self._raw_inputs = None
 
         try:
-            decoded = self.decode_input_args(
-                saved_state[BundleKeys.INPUTS_PARSED])
+            decoded = self.decode_input_args(saved_state[BundleKeys.INPUTS_PARSED])
             self._parsed_inputs = utils.AttributesFrozendict(decoded)
         except KeyError:
             self._parsed_inputs = None
@@ -597,8 +579,7 @@ class Process(
         elif state_label == process_states.ProcessState.WAITING:
             call_with_super_check(self.on_wait, state.data)
         elif state_label == process_states.ProcessState.FINISHED:
-            call_with_super_check(self.on_finish, state.result,
-                                  state.successful)
+            call_with_super_check(self.on_finish, state.result, state.successful)
         elif state_label == process_states.ProcessState.KILLED:
             call_with_super_check(self.on_kill, state.msg)
         elif state_label == process_states.ProcessState.EXCEPTED:
@@ -622,14 +603,10 @@ class Process(
             from_label = from_state.LABEL.value if from_state is not None else None
             try:
                 self._communicator.broadcast_send(
-                    body=None,
-                    sender=self.pid,
-                    subject='state_changed.{}.{}'.format(
-                        from_label, self.state.value))
+                    body=None, sender=self.pid, subject='state_changed.{}.{}'.format(from_label, self.state.value))
             except ConnectionClosed:
-                self.logger.info(
-                    'no connection available to broadcast state change from {} to {}'.
-                        format(from_label, self.state.value))
+                self.logger.info('no connection available to broadcast state change from {} to {}'.format(
+                    from_label, self.state.value))
 
     def on_exiting(self):
         state = self.state
@@ -646,8 +623,7 @@ class Process(
         # Input/output
         self._check_inputs(self._raw_inputs)
         raw_inputs = dict(self.raw_inputs) if self.raw_inputs else {}
-        self._parsed_inputs = self.create_input_args(self.spec().inputs,
-                                                     raw_inputs)
+        self._parsed_inputs = self.create_input_args(self.spec().inputs, raw_inputs)
 
         # Set up a process ID
         self._uuid = uuid.uuid4()
@@ -676,8 +652,7 @@ class Process(
         pass
 
     def on_output_emitted(self, output_port, value, dynamic):
-        self.__event_helper.fire_event(ProcessListener.on_output_emitted, self,
-                                       output_port, value, dynamic)
+        self.__event_helper.fire_event(ProcessListener.on_output_emitted, self, output_port, value, dynamic)
 
     @super_check
     def on_wait(self, awaitables):
@@ -730,25 +705,22 @@ class Process(
             try:
                 self._check_outputs()
             except ValueError:
-                raise StateEntryFailed(process_states.ProcessState.FINISHED,
-                                       result, False)
+                raise StateEntryFailed(process_states.ProcessState.FINISHED, result, False)
 
         self.future().set_result(self.outputs)
 
     @super_check
     def on_finished(self):
         """ Entered the FINISHED state """
-        self._fire_event(ProcessListener.on_process_finished,
-                         self.future().result())
+        self._fire_event(ProcessListener.on_process_finished, self.future().result())
 
     @super_check
     def on_except(self, exc_info):
-        self.future().set_exc_info(exc_info)
+        self.future().set_exception(exc_info[1])
 
     @super_check
     def on_excepted(self):
-        self._fire_event(ProcessListener.on_process_excepted,
-                         str(self.future().exception()))
+        self._fire_event(ProcessListener.on_process_excepted, str(self.future().exception()))
 
     @super_check
     def on_kill(self, msg):
@@ -770,8 +742,8 @@ class Process(
 
     # endregion
 
-    @tornado.gen.coroutine
-    def message_receive(self, msg):
+    @gen.coroutine
+    def message_receive(self, _comm, msg):
         intent = msg[process_comms.INTENT_KEY]
 
         if intent == process_comms.Intent.PLAY:
@@ -787,10 +759,11 @@ class Process(
         else:
             raise RuntimeError("Unknown intent")
 
-        if isinstance(result, futures.Future):
+        if concurrent.is_future(result):
+            # Wait for the process to actually finish
             result = yield result
 
-        raise tornado.gen.Return(result)
+        raise gen.Return(result)
 
     def close(self):
         """
@@ -801,14 +774,12 @@ class Process(
 
     # region State related methods
 
-    def transition_excepted(self, initial_state, final_state, exception,
-                            trace):
+    def transition_excepted(self, initial_state, final_state, exception, trace):
         # If we are creating, then reraise instead of failing.
         if final_state == process_states.ProcessState.CREATED:
             raise_(type(exception), exception, trace)
         else:
-            self.transition_to(process_states.ProcessState.EXCEPTED, exception,
-                               trace)
+            self.transition_to(process_states.ProcessState.EXCEPTED, exception, trace)
 
     def pause(self, msg=None):
         """
@@ -866,6 +837,7 @@ class Process(
             return futures.CancellableAction(do_pause, cookie=exception)
 
         elif isinstance(exception, process_states.KillInterruption):
+
             def do_kill(_next_state):
                 try:
                     # Ignore the next state
@@ -923,8 +895,7 @@ class Process(
         :param exception: The exception that caused the failure
         :param trace_back: Optional exception traceback
         """
-        self.transition_to(process_states.ProcessState.EXCEPTED, exception,
-                           trace_back)
+        self.transition_to(process_states.ProcessState.EXCEPTED, exception, trace_back)
 
     def kill(self, msg=None):
         """
@@ -959,8 +930,7 @@ class Process(
         # endregion
 
     def create_initial_state(self):
-        return self.get_state_class(process_states.ProcessState.CREATED)(
-            self, self.run)
+        return self.get_state_class(process_states.ProcessState.CREATED)(self, self.run)
 
     def recreate_state(self, saved_state):
         """
@@ -978,7 +948,7 @@ class Process(
     # region Execution related methods
 
     def run(self):
-        return self._run()
+        pass
 
     def execute(self):
         """
@@ -1059,8 +1029,7 @@ class Process(
         port_name = namespace.pop()
 
         if namespace:
-            port_namespace = self.spec().outputs.get_port(
-                namespace_separator.join(namespace))
+            port_namespace = self.spec().outputs.get_port(namespace_separator.join(namespace))
         else:
             port_namespace = self.spec().outputs
 
@@ -1098,9 +1067,7 @@ class Process(
                 if port.has_default():
                     port_value = port.default
                 elif port.required:
-                    raise ValueError(
-                        'Value not supplied for required inputs port {}'.
-                            format(name))
+                    raise ValueError('Value not supplied for required inputs port {}'.format(name))
                 else:
                     continue
             else:
@@ -1158,8 +1125,7 @@ class Process(
 
     def _check_outputs(self):
         # Check that the necessary outputs have been emitted
-        wrapped = utils.wrap_dict(
-            self._outputs, separator=self.spec().namespace_separator)
+        wrapped = utils.wrap_dict(self._outputs, separator=self.spec().namespace_separator)
         for name, port in self.spec().outputs.items():
             valid, msg = port.validate(wrapped.get(name, ports.UNSPECIFIED))
             if not valid:
